package com.fuyuanshen.modules.system.service.app;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fuyuanshen.exception.BadRequestException;
import com.fuyuanshen.modules.system.domain.Device;
import com.fuyuanshen.modules.system.domain.DeviceType;
import com.fuyuanshen.modules.system.domain.app.APPDevice;
import com.fuyuanshen.modules.system.domain.app.APPDeviceType;
import com.fuyuanshen.modules.system.domain.dto.DeviceQueryCriteria;
import com.fuyuanshen.modules.system.domain.dto.app.APPUnbindDTO;
import com.fuyuanshen.modules.system.enums.BindingStatusEnum;
import com.fuyuanshen.modules.system.enums.UserType;
import com.fuyuanshen.modules.system.mapper.DeviceMapper;
import com.fuyuanshen.modules.system.mapper.DeviceTypeMapper;
import com.fuyuanshen.modules.system.mapper.app.APPDeviceMapper;
import com.fuyuanshen.utils.PageResult;
import com.fuyuanshen.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @Description:
 * @Author: WY
 * @Date: 2025/5/16
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class APPDeviceServiceImpl extends ServiceImpl<APPDeviceMapper, APPDevice> implements APPDeviceService {

    private final APPDeviceMapper appDeviceMapper;
    private final DeviceMapper deviceMapper;
    private final DeviceTypeMapper deviceTypeMapper;


    /**
     * APP/小程序用户设备绑定
     *
     * @param mac
     */
    @Override
    @Transactional
    public void appBindDevice(String mac) {
        List<Device> devices = deviceMapper.selectList(new QueryWrapper<Device>().eq("device_mac", mac));
        if (CollectionUtil.isEmpty(devices)) {
            throw new BadRequestException("请先将设备入库！！！");
        }
        List<APPDevice> appDevices = appDeviceMapper.selectList(new QueryWrapper<APPDevice>()
                .eq("device_mac", mac).eq("binding_type", UserType.APP.getValue()));
        if (CollectionUtil.isNotEmpty(appDevices)) {
            throw new BadRequestException("该设备已绑定！！！");
        }

        Device device = devices.get(0);
        APPDevice appDevice = new APPDevice();
        BeanUtil.copyProperties(appDevice, device);
        appDevice.setBindingType(UserType.APP.getValue());
        appDevice.setBindingStatus(BindingStatusEnum.BOUND.getCode());
        Long currentUserId = SecurityUtils.getCurrentUserId();
        appDevice.setCustomerId(currentUserId);
        appDeviceMapper.insert(appDevice);

        DeviceType deviceType = deviceTypeMapper.selectById(device.getDeviceType());
        APPDeviceType appDeviceType = new APPDeviceType();
        BeanUtil.copyProperties(appDeviceType, deviceType);
        deviceTypeMapper.insert(appDeviceType);

    }


    /**
     * 分页查询APP/小程序设备
     *
     * @param criteria
     * @param page
     * @return
     */
    @Override
    public PageResult<APPDevice> queryAll(Page<APPDevice> page, DeviceQueryCriteria criteria) {
        IPage<APPDevice> devices = appDeviceMapper.queryAll(page, criteria);
        return new PageResult<>(devices.getRecords(), devices.getTotal());
    }


    /**
     * APP/小程序用户设备解绑
     *
     * @param deviceForm
     */
    @Override
    @Transactional
    public void unbindAPPDevice(APPUnbindDTO deviceForm) {

        appDeviceMapper.delete(new QueryWrapper<APPDevice>().eq("device_mac", deviceForm.getDeviceMac()));
        Device device = new Device();
        device.setId(deviceForm.getCustomerId());
        device.setBindingStatus(BindingStatusEnum.UNBOUND.getCode());
        deviceMapper.updateById(device);
    }


}
